@@ -28,9 +28,9 @@ const double FLAN_SimClone::DEATH_EPS_SIM=1.e-4;
 NumericVector FLAN_SimClone::computeSample(int n) {
 
   std::string name=mDist->getDistName();
-  
+
   NumericVector sample=rexp(n,mFitness);
-  
+
 
   if(name.compare("dirac") == 0){
 
@@ -69,7 +69,7 @@ NumericVector FLAN_SimClone::computeSample(int n) {
   //         for (int k=0;k<n;k++) {
 	for(NumericVector::iterator it = sample.begin();it != sample.end(); ++it) {
 	      p=exp(-(*it));
-	      
+
 	      geoD=rgeom(1,p)[0];
 
 	      if(geoD>=0) *it=geoD+1;
@@ -79,18 +79,18 @@ NumericVector FLAN_SimClone::computeSample(int n) {
 	for(NumericVector::iterator it = sample.begin();it != sample.end(); ++it) {
 
 	    p=exp(-(*it));
-	    
+
 	    up=(1-2*mDeath)/(1-mDeath*(1+p));
-	    
+
 	    beD=rbinom(1,1,up)[0];
-	    
+
 	    if (beD==1){
 
 	      geoD=rgeom(1,p*up)[0];
-	      
+
 	      if(geoD>=0) *it=geoD+1;
 	      else if (geoD<0 || geoD!=geoD) *it=-1.e-5;
-	      
+
 	    } else *it=0;
 	}
       }
@@ -128,7 +128,7 @@ int FLAN_SimClone::splitTimes(double t) {
     //division times of current generation
     std::vector<double> st (1);
     if(name.compare("lnorm") == 0) st=as< std::vector<double> >(rlnorm(1,params[0],params[1]));
-    
+
     if(name.compare("gamma") == 0) st=as< std::vector<double> >(rgamma(1,params[0],params[1]));
 
     // take only the times if before t
@@ -136,7 +136,7 @@ int FLAN_SimClone::splitTimes(double t) {
     // number of alive cells
     int ng=0, nActiveAlives;
     std::vector<double> gtf;
-    
+
     // If no cells death
     if(mDeath<DEATH_EPS_SIM){
       if(st[0]<t){
@@ -155,7 +155,7 @@ int FLAN_SimClone::splitTimes(double t) {
 	gtf.resize(2*ng);
 	if(name.compare("lnorm") == 0) gtf=as< std::vector<double> >(rlnorm(2*ng,params[0],params[1]));
 	if(name.compare("gamma") == 0) gtf=as< std::vector<double> >(rgamma(2*ng,params[0],params[1]));
-	
+
 	//  split times of daughters with duplicating split times
         for (int i=0;i<ng;i++) {
             gtf[2*i]+=st[i];
@@ -166,7 +166,7 @@ int FLAN_SimClone::splitTimes(double t) {
         // keep only values less than t
         // daughters that still divide
         // keep their split times
-	
+
 	nActiveAlives=0;
 	st.resize(ng);
         for (int i=0;i<ng;i++) {
@@ -186,7 +186,7 @@ int FLAN_SimClone::splitTimes(double t) {
       }
     // If cells death with prob mDeath
     } else {
-    
+
       NumericVector u=runif(1,0.,1.);
       if((st[0]<t) && (u[0]>mDeath)){
         ng=1;
@@ -199,14 +199,14 @@ int FLAN_SimClone::splitTimes(double t) {
 	ng=0;
       }
       nActiveAlives=ng;
-      
+
       while ((ng>0) && (ng<1.e+6)) {
 	g++;
 // 	NumericVector gtf;
 	gtf.resize(2*ng);
 	if(name.compare("lnorm") == 0) gtf=as< std::vector<double> >(rlnorm(2*ng,params[0],params[1]));
 	if(name.compare("gamma") == 0) gtf=as< std::vector<double> >(rgamma(2*ng,params[0],params[1]));
-	
+
 	//  split times of daughters with duplicating split times
         for (int i=0;i<ng;i++) {
             gtf[2*i]+=st[i];
@@ -214,9 +214,9 @@ int FLAN_SimClone::splitTimes(double t) {
         }
 
         ng*=2;
-	
+
 	u=runif(ng,0.,1.);
-	
+
 	nActiveAlives=0;
 	st.resize(ng);
 	for (int i=0;i<ng;i++) {
@@ -250,28 +250,54 @@ const double FLAN_Clone::DEATH_EPS_DIST=1.e-4;
 
 
 // create object for GF method
-
+//
+// double FLAN_ExponentialClone::testIntegral(double a, double b){
+//   MATH_Params params;
+//   params.rho=mFitness;
+//   double d1=mDeath/(1-mDeath);
+//   params.delta=d1;
+//
+//   mIntegrator->setFunction("TEST",&params);
+//   //integrate the function in [0,1]
+//   double I;
+//   // m=0 probability
+//   // std::cout<<"Call of computeIntegral"<<std::endl;
+//   I=mIntegrator->computeIntegral(a,b);
+//
+//   return I;
+// }
 
 NumericVector FLAN_ExponentialClone::computeProbability(int m){
 
+//   std::cout<<"Compute CloneExp probabilities"<<std::endl;
+
   std::vector<double> P(m+1);
 
   if(mDeath<DEATH_EPS_DIST){
+//     std::cout<<"Compute CloneExp probabilities (death=0)"<<std::endl;
 
     P[0]=0;
-
     if (m > 0){
       int k=1;
       for(std::vector<double>::iterator it=P.begin()+1 ; it!= P.end() ; ++it,k++) *it=mFitness*R::beta(mFitness+1,k);
     }
   } else {
+//         std::cout<<"Compute CloneExp probabilities (death>0)"<<std::endl;
+
     double d1=mDeath/(1-mDeath);
     int m_max=1000;
-    mIntegrator->setFunctionName("CLONE_P0_WD");
+    // std::cout<<"Set the integrand and the parameters"<<std::endl;
+    MATH_Params params;
+    params.rho=mFitness;
+    params.delta=d1;
+    params.k=0.;
+    mIntegrator->setFunction("CLONE_P0_WD",&params);
     //integrate the function in [0,1]
     double I;
     // m=0 probability
-    I=mIntegrator->integralFunction(0.,1.,mFitness,d1,0.);
+    // std::cout<<"Call of computeIntegral"<<std::endl;
+    I=mIntegrator->computeIntegral(0.,1.);
+    // std::cout<<"DONE : I ="<<I<<std::endl;
     P[0]=I*d1*mFitness;
 
     if(m > 0){
@@ -282,10 +308,14 @@ NumericVector FLAN_ExponentialClone::computeProbability(int m){
       //m>0 probability
       int m1=m;
       if (m1>=m_max) m1=m_max;
-      mIntegrator->setFunctionName("CLONE_PK_WD");
       std::vector<double>::iterator it=P.begin()+1;
+      // MATH_Params params;
+      // params.rho=mFitness;
+      // params.delta=d1;
       for (int k=1;k<=m1;k++,++it) {
-	    I=mIntegrator->integralFunction(0.,1,mFitness,d1,k);
+      params.k=k;
+      mIntegrator->setFunction("CLONE_PK_WD",&params);
+	    I=mIntegrator->computeIntegral(0.,1);
 	    *it=I*d2*mFitness;
       }
 
@@ -306,9 +336,9 @@ List FLAN_ExponentialClone::computeProbability1DerivativeRho(int m){
   std::vector<double> P(m+1);
   std::vector<double> dP_dr(m+1);
 //   double pk;
-  
+
   if(mDeath<DEATH_EPS_DIST){
-    
+
     P[0]=0;
     dP_dr[0]=0;
     if(m == 0) return List::create(_["P"]=P[0],_["dP_dr"]=dP_dr[0]);
@@ -323,24 +353,28 @@ List FLAN_ExponentialClone::computeProbability1DerivativeRho(int m){
     }
   } else {
     double d1=mDeath/(1-mDeath);
-    
+
     double I;
-    
+
     int m_max=1000;
     // set the function type
-    mIntegrator->setFunctionName("CLONE_P0_WD");
-    I=mIntegrator->integralFunction(0.,1.,mFitness,d1,0.);
+    MATH_Params params;
+    params.rho=mFitness;
+    params.delta=d1;
+    params.k=0.;
+    mIntegrator->setFunction("CLONE_P0_WD",&params);
+    I=mIntegrator->computeIntegral(0.,1.);
     P[0]=I*d1*mFitness;
-    
+
     mIntegrator->setFunctionName("CLONE_dP0_dr_WD");
     //integrate the function in [0,1]
     // m=0 probability
-    I=mIntegrator->integralFunction(0.,1.,mFitness,d1,0.);
+    I=mIntegrator->computeIntegral(0.,1.);
     dP_dr[0]=I*d1*mFitness+P[0]/mFitness;
 
     if (m==0) return List::create(_["P"]=P[0],_["dP_dr"]=dP_dr[0]);
 
-    
+
     double d2=(1.-2.*mDeath)/(1-mDeath);
     d2*=d2;
     //m>0 probability
@@ -349,12 +383,13 @@ List FLAN_ExponentialClone::computeProbability1DerivativeRho(int m){
     std::vector<double>::iterator itP=P.begin()+1 ;
     std::vector<double>::iterator itdP=dP_dr.begin()+1 ;
     for (int k=1;k<=m1;k++,++itP,++itdP) {
-	mIntegrator->setFunctionName("CLONE_PK_WD");
-	I=mIntegrator->integralFunction(0.,1,mFitness,d1,k);
+      params.k=k;
+	mIntegrator->setFunction("CLONE_PK_WD",&params);
+	I=mIntegrator->computeIntegral(0.,1);
 	*itP=I*d2*mFitness;
-	
+
 	mIntegrator->setFunctionName("CLONE_dPK_dr_WD");
-	I=mIntegrator->integralFunction(0.,1,mFitness,d1,k);
+	I=mIntegrator->computeIntegral(0.,1.);
         *itdP=(*itP)/mFitness+I*d2*mFitness;
     }
 
@@ -370,7 +405,7 @@ List FLAN_ExponentialClone::computeProbability1DerivativeRho(int m){
       kd=(double)(k);
       kpr=pow(kd,-mFitness-1);
       *itP=a*kpr;
-      
+
       *itdP=b*kpr*(
             gfn*(c-mFitness*log(kd))
             +rhodg);
@@ -384,58 +419,65 @@ List FLAN_ExponentialClone::computeProbability1DerivativeRho(int m){
 
 
 std::vector<double> FLAN_ExponentialClone::computeGeneratingFunction2(double rho,std::vector<double> Z){
-  
+
   double eps =1e-8;
-  
+
   std::vector<double> H(Z.size());
   std::vector<double>::iterator itH=H.begin();
-  
+
   for (std::vector<double>::iterator itZ = Z.begin() ; itZ != Z.end() ; ++itZ, ++itH){
     if(fabs(*itZ) <= eps) *itH=0.;
     else if(fabs(1-(*itZ)) <= eps) *itH=1.;
     else {
       double dstar,zstar,I;
-      
+
       dstar=mDeath/(1-mDeath);
       zstar=((*itZ)-dstar)/(1-(*itZ));
-      
-      mIntegrator->setFunctionName("CLONE_PGF");
-      I=mIntegrator->integralFunction(0.,1.,rho,zstar,0);
-      
+      MATH_Params params;
+      params.rho=rho;
+      params.delta=zstar;
+      params.k=0.;
+      mIntegrator->setFunction("CLONE_PGF",&params);
+      I=mIntegrator->computeIntegral(0.,1.);
+
       *itH=dstar+I*zstar*(1-dstar)*rho;
-      
+
 
     }
   }
-  
+
   return H;
-    
+
 }
 
 
 double FLAN_ExponentialClone::computeGeneratingFunction1DerivativeRho(double z){
-  
+
   double eps =1e-8;
-  
+
   if(fabs(z) <= eps) return 0.;
   else if(fabs(1-z) <= eps) return 0.;
   else {
     double dstar,zstar,I1,I2,h;
-    
+
     dstar=mDeath/(1-mDeath);
     zstar=(z-dstar)/(1-z);
-    
-    mIntegrator->setFunctionName("CLONE_PGF");
-    I1=mIntegrator->integralFunction(0.,1.,mFitness,zstar,0);
-    
-    mIntegrator->setFunctionName("CLONE_PGF_dr");
-    I2=mIntegrator->integralFunction(0.,1.,mFitness,zstar,0);
-    
+    MATH_Params params;
+    params.rho=mFitness;
+    params.delta=zstar;
+    params.k=0.;
+
+    mIntegrator->setFunction("CLONE_PGF",&params);
+    I1=mIntegrator->computeIntegral(0.,1.);
+
+    mIntegrator->setFunctionName("CLONE_dPGF_dr");
+    I2=mIntegrator->computeIntegral(0.,1.);
+
     h=zstar*(1-dstar)*(I1+I2*mFitness);
-    
+
     return h;
   }
-  
+
 }
 
 
@@ -455,19 +497,23 @@ NumericVector FLAN_DiracClone::computeProbability(int m){
     P[0]=0;
 
     if(m > 0){
+      int index=1;
       int n=floor(log((double)(m))/log(2.));
         //take only the indices index=2^ind for ind in [0,n] where index < m
-      for (int k=0;k<=n;k++) P[pow(2.,k)]=(1-pow(2.,-mFitness))*pow(2.,-k*mFitness);
+      for (int k=0;k<=n;k++) {
+	P[index]=(1-pow(2.,-mFitness))*pow(2.,-k*mFitness);
+	index*=2;
+      }
     }
 
-  } else {   
-        
+  } else {
+
     // case with death >0
-    double eps=1.e-6;
+    // double eps=1.e-6;
     double deps=1.e-8;
-    
-    int itmax=25;
-  
+
+    int itmax=19;  // Arbitrary choice
+
     // initiate the polynome X
     mPol.setDegree(1);
 
@@ -478,32 +524,33 @@ NumericVector FLAN_DiracClone::computeProbability(int m){
     double t=exp(-mFitness*a);
     double ti=1;//t^i
     int Pol_degree=1;
-    
+
     double sumP;
-    double sumP_old;
-    
+    // double sumP_old;
+
 
     // initialize P
     P[0]=0;P[1]=1;
     sumP=1;
-    double err = 1;
+    // double err = 1;
     int dmax;
     int k;
     // loop
     std::vector<double>::iterator itP;
-    while (((Pol_degree<=m) || (err>eps)) && (i<itmax)) {
+    // while (((Pol_degree<=m) || (err>eps)) && (i<itmax)) {
+    while(i < itmax){
 	// next iteration
 	i++;
 	ti*=t;
-	sumP_old=sumP;
-	
+	// sumP_old=sumP;
+
 	// update P
 
 	mPol.square_fft();
 
 	mPol*=umd;
 	mPol+=mDeath;
-	
+
 	mPol.reduce(deps);
 	Pol_degree=mPol.getDegree();
 	// update pk=sum_{i=1}^{i=number of polynoms computer} Pi[k] t^i
@@ -514,14 +561,15 @@ NumericVector FLAN_DiracClone::computeProbability(int m){
 	    *itP+=mPol[k]*ti;
 	    sumP+=*itP;
 	}
-      
+
 	// next iteration
-	err = fabs(1-sumP_old/sumP);
+	// err = fabs(1-sumP_old/sumP);
+  // std::cout<<"écart relatif (it n°"<<i<<") ="<<err<<std::endl;
     } //end loop
 
     // multiply by (1-t)
     for(itP=P.begin() ; itP!=P.end();++itP) (*itP)*=(1-t);
-    
+
   }
 
   return NumericVector(P.begin(),P.end());
@@ -533,33 +581,34 @@ List FLAN_DiracClone::computeProbability1DerivativeRho(int m){
 
   std::vector<double> P(m+1);
   std::vector<double> dP_dr(m+1);
-  
+
   if(mDeath<DEATH_EPS_DIST){
 
     P[0]=0;
     dP_dr[0]=0;
     if(m == 0) return List::create(_["P"]=P[0],_["dP_dr"]=dP_dr[0]);
-    
+
 
     int n=floor(log((double)(m))/log(2.));
-    double index;
+    int index=1;
 
     for(int k=0;k<=n;k++) {
-      index=pow(2.,k);
+//       index=(int)(pow(2,k));
       P[index]=(1-pow(2.,-mFitness))*pow(2.,-k*mFitness);
       dP_dr[index]=log(2.)*(pow(2.,-mFitness*(k+1))-k*P[index]);
+      index*=2;
     }
 
   } else {
      // case with death >0
-    double eps=1.e-6;
+    // double eps=1.e-6;
     double deps=1.e-8;
-    
-    int itmax=25;
-  
+
+    int itmax=20;
+
     // initiate the polynome X
     mPol.setDegree(1);
-    
+
 
     // minimum number of iterations
     int i=0;
@@ -570,64 +619,64 @@ List FLAN_DiracClone::computeProbability1DerivativeRho(int m){
     double ti=1;//t^i
     double iti=0; // i*t^i
     int Pol_degree=1;
-    
+
     double sumP;
-    double sumP_old;
-    
+    // double sumP_old;
+
 
     // initialize P
     P[0]=0;P[1]=1;
     dP_dr[0]=0;dP_dr[1]=t-umt;
     sumP=1;
-    double err = 1;
+    // double err = 1;
     int dmax;
     int k;
     // loop
     std::vector<double>::iterator itP;
     std::vector<double>::iterator itdP;
-    while (((Pol_degree<=m) || (err>eps)) && (i<itmax)) {
+    // while (((Pol_degree<=m) || (err>eps)) && (i<itmax)) {
+    while(i < itmax){
 	// next iteration
 	i++;
 	ti*=t;
 	iti=i*ti;
-	sumP_old=sumP;
-		  
+	// sumP_old=sumP;
+
 	// update P
 	mPol.square_fft();
-	  
+
 	// update P
 	mPol*=(1-mDeath);
 	mPol+=mDeath;
 	mPol.reduce(deps);
-      
+
 	Pol_degree=mPol.getDegree();
 
 	dmax=(m<Pol_degree)?m:Pol_degree;
 	sumP=0;
-	
+
 	itP=P.begin();
 	itdP=dP_dr.begin();
 	for (k=0;k<=dmax;k++,++itP,++itdP) {
-	    
+
 	  *itP+=mPol[k]*ti;
 	  sumP+=*itP;
-	  
+
 	  *itdP-=mPol[k]*iti;
 	}
-	  
-      err = fabs(1-sumP_old/sumP);
-      
+
+      // err = fabs(1-sumP_old/sumP);
     }
-    
+
     itdP=dP_dr.begin();
     for(itP=P.begin() ; itP!=P.end() ; ++itP,++itdP){
       (*itdP)*=umt;
       (*itdP)+=t*(*itP);
       (*itP)*=umt;
     }
-      
+
   }
-  
+
   return List::create(_["P"]=NumericVector(P.begin(),P.end()),
 		      _["dP_dr"]=NumericVector(dP_dr.begin(),dP_dr.end()));
 
@@ -636,7 +685,7 @@ List FLAN_DiracClone::computeProbability1DerivativeRho(int m){
 
 std::vector<double> FLAN_DiracClone::computeGeneratingFunction2(double rho,std::vector<double> Z){
 
-  
+
     std::vector<double> H (Z.size());
     std::vector<double>::iterator itH=H.begin();
     double eps=1.e-8;
@@ -676,7 +725,7 @@ std::vector<double> FLAN_DiracClone::computeGeneratingFunction2(double rho,std::
       }
       *itH=s;
     }
-    
+
   return H;
 
 }
