@@ -33,7 +33,7 @@ private:
 
   std::string mName;                // Name of the distribution
   std::vector<double> mParams;      // Parameter(s) of the distribution
-  
+
 public:
 
   FLAN_Dist(){};
@@ -79,40 +79,45 @@ private:
 
     double mFitness;
     double mDeath;
-    FLAN_Dist *mDist=NULL;   // Lifetime distribution
-    FLAN_Dist *mDistN=NULL;  // Lifetime distribution for normal cells (drawing function)
+    FLAN_Dist *mDist;   // Lifetime distribution
+    FLAN_Dist *mDistN;  // Lifetime distribution for normal cells (drawing function)
 
 protected:
   static const double DEATH_EPS_SIM;     // Threshold for death
 
 public:
-    FLAN_SimClone(){};
+    FLAN_SimClone(){
+      if(!mDist) delete mDist;
+      mDist=NULL;
+      if(!mDistN) delete mDistN;
+      mDistN=NULL;
+    };
 
     ~FLAN_SimClone(){
-      
       if(!mDist) delete mDist;
       if(!mDistN) delete mDistN;
-      
     };
 
     FLAN_SimClone(double rho,double death, List dist){
 
       mFitness=rho;
       mDeath=death;
-      
+
       if(!mDist) delete mDist;
       mDist= new FLAN_Dist(dist);
+      if(!mDistN) delete mDistN;
+      mDistN=NULL;
 
     };
-    
+
     FLAN_SimClone(double rho,double death, List distn, List distm){
 
       mFitness=rho;
       mDeath=death;
-      
+
       if(!mDist) delete mDist;
       if(!mDistN) delete mDistN;
-      
+
       mDist= new FLAN_Dist(distm);
       mDistN= new FLAN_Dist(distn);
 
@@ -122,7 +127,12 @@ public:
 
       mFitness=rho;
       mDeath=death;
+
+      if(!mDist) delete mDist;
+      if(!mDistN) delete mDistN;
+
       mDist=dist;
+      mDistN=NULL;
 
     };
 
@@ -131,9 +141,9 @@ public:
     // ------------------------
 
     NumericVector computeSample(int n);
-    
+
     int splitTimes(double t);
-    
+
 
 };
 
@@ -146,27 +156,29 @@ public:
 class FLAN_Clone {
 
 private:
-  
+
 
 protected:
     static const double DEATH_EPS_DIST;     // Threshold for death
     double mFitness;    // Realtive fitness
     double mDeath;      // Death probability
-    
+//     double mPlateff;
+
     FLAN_Clone(){};
 
     // create object for GF method
-    
+
     FLAN_Clone(List params){
-      
+
       mDeath=as<double>(params["death"]);
-      
-      if(params.size() > 2) {
+
+      if(params.size() >= 2) {
 	mFitness=as<double>(params["fitness"]);
       }
-        
+      // std::cout<<"Fitness ="<<mFitness<<std::endl;
+      // std::cout<<"Death ="<<mDeath<<std::endl;
     };
-    
+
     FLAN_Clone(double death){
       mDeath=death;
     };
@@ -175,9 +187,14 @@ protected:
     FLAN_Clone(double rho,double death){
       mFitness=rho;
       mDeath=death;
-      
     };
 
+//     FLAN_Clone(double rho,double death,double plateff){
+//       mFitness=rho;
+//       mDeath=death;
+//       mPlateff=plateff;
+//     };
+
 
     ~FLAN_Clone(){};
 
@@ -206,7 +223,7 @@ public:
 
     virtual NumericVector computeProbability(int m) = 0;
 
-    
+
     virtual List computeProbability1DerivativeRho(int m) = 0;
 
 
@@ -220,9 +237,9 @@ public:
       Z[0]=z;
       return computeGeneratingFunction2(mFitness,Z)[0];
     };
-    
+
     virtual std::vector<double> computeGeneratingFunction2(double rho,std::vector<double> Z) = 0;
-    
+
     virtual double computeGeneratingFunction1DerivativeRho(double z) = 0;
 
 };
@@ -240,41 +257,58 @@ class FLAN_ExponentialClone : public FLAN_Clone {
 
   private:
 
+    MATH_Integration* mIntegrator;
 
-    MATH_Integration* mIntegrator=NULL;
-
-    void init(List fns) {
+    // void init(List fns) {
+    void init() {
+//       std::cout<<"Call init() function in CloneExp constructor"<<std::endl;
       List info=Environment::base_namespace().get(".Machine");
       double flantol=info["double.eps"];
       flantol=sqrt(flantol);
       int flansubd=1000;
-      
+
       if(!mIntegrator) delete mIntegrator;
-      mIntegrator=new MATH_Integration(fns,flantol,flansubd);
+      // mIntegrator=new MATH_Integration(fns,flantol,flansubd);
+      mIntegrator=new MATH_Integration(flantol,flansubd);
     }
 
   public:
 
-    FLAN_ExponentialClone():FLAN_Clone(){};
-    
+    FLAN_ExponentialClone():FLAN_Clone(){
+      init();
+    };
+
     FLAN_ExponentialClone(List params):FLAN_Clone(params) {
-      List fns=params["integrands"];
-      init(fns);
+      // List fns=params["integrands"];
+      init();
+      // std::cout<<"fitness ="<<mFitness<<std::endl;
+      // std::cout<<"death ="<<mDeath<<std::endl;
     };
-    FLAN_ExponentialClone(double death,List fns):FLAN_Clone(death) {
-      init(fns);
+    // FLAN_ExponentialClone(double death):FLAN_Clone(death) {
+      // init(fns);
+    FLAN_ExponentialClone(double death):FLAN_Clone(death) {
+    // FLAN_ExponentialClone(double rho,double death,List fns):FLAN_Clone(rho,death) {
+      init();
     };
-    FLAN_ExponentialClone(double rho,double death,List fns):FLAN_Clone(rho,death) {
-      init(fns);
+    FLAN_ExponentialClone(double rho,double death):FLAN_Clone(rho,death) {
+//       	std::cout<<"Constructor of ExpClone"<<std::endl;
+// 	std::cout<<"fitness(clone) ="<<mFitness<<std::endl;
+// 	std::cout<<"death(clone) ="<<mDeath<<std::endl;
+
+      init();
+
     };
     ~FLAN_ExponentialClone(){
       if(!mIntegrator) delete mIntegrator;
     };
 
+
+    double testIntegral(double a, double b);
+
     NumericVector computeProbability(int m);
-    
+
     List computeProbability1DerivativeRho(int m) ;
-    
+
     std::vector<double> computeGeneratingFunction2(double rho,std::vector<double> Z);
 
     double computeGeneratingFunction1DerivativeRho(double z)  ;
@@ -290,13 +324,13 @@ class FLAN_ExponentialClone : public FLAN_Clone {
 class FLAN_DiracClone : public FLAN_Clone {
 
 private:
-  
+
   void init_death(){
       mPol=MATH_Polynom();
     };
 
 protected:
-    
+
     MATH_Polynom mPol;
 
 public:
@@ -314,9 +348,9 @@ public:
     ~FLAN_DiracClone(){};
 
     NumericVector computeProbability(int m);
-    
+
     List computeProbability1DerivativeRho(int m);
-    
+
     std::vector<double> computeGeneratingFunction2(double rho,std::vector<double> Z);
 
     double computeGeneratingFunction1DerivativeRho(double z)  ;
